//-------------------------------------------------------------------------------------------
// PRNG is Mersenne Twister implementation MT19937
// http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html
// A fast random number generator that is very good for Monte-Carlo simulations
// but not random enough for cryptographic use
//-------------------------------------------------------------------------------------------
#ifndef __TWISTER_RANDOM_H
#define __TWISTER_RANDOM_H
//-------------------------------------------------------------------------------------------

#include <stdio.h>
#include <stdint.h>
#include <math.h>

//-------------------------------------------------------------------------------------------

#define RANDOM_MT19937_LENGTH_N 624
#define RANDOM_MT19937_LENGTH_M 397
#define RANDOM_MT19937_MATRIX_A 0x9908B0DFUL
#define RANDOM_MT19937_UPPER_MASK 0x80000000UL
#define RANDOM_MT19937_LOWER_MASK 0x7fffffffUL

//-------------------------------------------------------------------------------------------

class Random
{
	public:
		~Random();

		static Random *instance();

		// generates a random number on [0,0xffffffff]-interval
		uint32_t randomUInt32();
		// generates a random number on [0,0x7fffffff]-interval
		int32_t randomInt31();

		// generates a random number on [0,1]-real-interval
		double randomReal1();

		void reset();
		
		void seed(int32_t seedValue);
		void seed(uint32_t seedValue);
		void seed(const int32_t *key,int keyLength);
		void seed(const uint32_t *key,int keyLength);

	private:
		
		static Random *m_instance;
		
		uint32_t m_mt[RANDOM_MT19937_LENGTH_N];
		int32_t m_mti;
		
		Random();

		void initializePRNG(uint32_t s);
		void initializePRNGArray(const uint32_t *key,int keyLength);
		
		uint32_t generateRandomUInt32();
};

//-------------------------------------------------------------------------------------------
#endif
//-------------------------------------------------------------------------------------------

